@@ -1,6 +1,5 @@
 #!/usr/bin/env tclsh
 # mpkgall.tcl
-# $Id: mpkgall.tcl 79597 2011-06-19 20:59:11Z jmr@macports.org $
 #
 # Copyright (c) 2003 Kevin Van Vechten <kevin@opendarwin.org>
 # Copyright (c) 2002 Apple Inc.
@@ -10,14 +9,14 @@
 # modification, are permitted provided that the following conditions
 # are met:
 # 1. Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer.
+#	 notice, this list of conditions and the following disclaimer.
 # 2. Redistributions in binary form must reproduce the above copyright
-#    notice, this list of conditions and the following disclaimer in the
-#    documentation and/or other materials provided with the distribution.
+#	 notice, this list of conditions and the following disclaimer in the
+#	 documentation and/or other materials provided with the distribution.
 # 3. Neither the name of Apple Inc. nor the names of its contributors
-#    may be used to endorse or promote products derived from this software
-#    without specific prior written permission.
-# 
+#	 may be used to endorse or promote products derived from this software
+#	 without specific prior written permission.
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -53,7 +52,7 @@ proc copy_package_if_available {portname basepath destpath} {
 	# XXX: don't overwrite Apple X11
 	# XXX: probably should exclude KDE here too
 	if {$portname == "XFree86"} { return {} }
-	
+
 	if {[catch {set res [mportsearch "^$portname\$"]} error]} {
 		puts stderr "Internal error: port search failed: $error"
 		return
@@ -63,11 +62,11 @@ proc copy_package_if_available {portname basepath destpath} {
 		if {![info exists portinfo(name)]} { return -1 }
 		if {![info exists portinfo(version)]} { return -1 }
 		if {![info exists portinfo(categories)]} { return -1 }
-		
+
 		set portname $portinfo(name)
 		set portversion $portinfo(version)
 		set category [lindex $portinfo(categories) 0]
-		
+
 		set depends {}
 		if {[info exists portinfo(depends_run)]} { eval "lappend depends $portinfo(depends_run)" }
 		if {[info exists portinfo(depends_lib)]} { eval "lappend depends $portinfo(depends_lib)" }
@@ -75,13 +74,13 @@ proc copy_package_if_available {portname basepath destpath} {
 		foreach depspec $depends {
 			set dep [lindex [split $depspec :] end]
 			set result [copy_package_if_available $dep $basepath $destpath]
-			if {$result == -1} { 
+			if {$result == -1} {
 				return -1
 			} else {
 				eval "lappend dependencies $result"
 			}
 		}
-		
+
 		set pkgname "${portname}-${portversion}.pkg"
 		lappend dependencies $pkgname
 		set pkgpath "${basepath}/${category}/${pkgname}"
@@ -95,14 +94,14 @@ proc copy_package_if_available {portname basepath destpath} {
 			return -1
 		}
 	}
-	
+
 	return $dependencies
 }
 
 proc write_description_plist {infofile portname portversion description} {
 	set infofd [open ${infofile} w+]
 	puts $infofd {<?xml version="1.0" encoding="UTF-8"?>
-<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
 <plist version="1.0">
 }
 	puts $infofd "<dict>
@@ -120,10 +119,10 @@ proc write_description_plist {infofile portname portversion description} {
 }
 
 proc write_welcome_html {filename portname portversion long_description description homepage} {
-    set fd [open ${filename} w+]
-    if {$long_description == ""} {
-	set long_description $description
-    }
+	set fd [open ${filename} w+]
+	if {$long_description eq ""} {
+		set long_description $description
+	}
 
 puts $fd "
 <html lang=\"en\">
@@ -137,15 +136,15 @@ puts $fd "
 <font face=\"Helvetica\">${long_description}</font>
 <p>"
 
-    if {$homepage != ""} {
-	puts $fd "<font face=\"Helvetica\">${homepage}</font><p>"
-    }
+	if {$homepage ne ""} {
+		puts $fd "<font face=\"Helvetica\">${homepage}</font><p>"
+	}
 
-    puts $fd "<font face=\"Helvetica\">This installer guides you through the steps necessary to install ${portname} ${portversion} for Mac OS X. To get started, click Continue.</font>
+	puts $fd "<font face=\"Helvetica\">This installer guides you through the steps necessary to install ${portname} ${portversion} for Mac OS X. To get started, click Continue.</font>
 </body>
 </html>"
 
-    close $fd
+	close $fd
 }
 
 proc write_PkgInfo {infofile} {
@@ -156,11 +155,11 @@ proc write_PkgInfo {infofile} {
 
 proc mpkg_write_info_plist {infofile portname portversion portrevision destination dependencies} {
 	set vers [split $portversion "."]
-	
-	if {[string index $destination end] != "/"} {
+
+	if {[string index $destination end] ne "/"} {
 		append destination /
 	}
-	
+
 	set depxml ""
 	foreach dep $dependencies {
 		append depxml "<dict>
@@ -174,7 +173,7 @@ proc mpkg_write_info_plist {infofile portname portversion portrevision destinati
 
 	set infofd [open ${infofile} w+]
 	puts $infofd {<?xml version="1.0" encoding="UTF-8"?>
-<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
 <plist version="1.0">
 }
 	puts $infofd "<dict>
@@ -206,9 +205,9 @@ proc mpkg_write_info_plist {infofile portname portversion portrevision destinati
 # Standard procedures
 
 proc fatal args {
-    global argv0
-    puts stderr "$argv0: $args"
-    exit
+	global argv0
+	puts stderr "$argv0: $args"
+	exit
 }
 
 # Main
@@ -217,120 +216,118 @@ array set variations [list]
 #	set ui_options(ports_verbose) yes
 
 if {[catch {mportinit ui_options options variations} result]} {
-    puts "Failed to initialize ports system, $result"
-    exit 1
+	puts "Failed to initialize ports system, $result"
+	exit 1
 }
 
 package require Pextlib
 
 # If no arguments were given, default to all ports.
 if {[llength $argv] == 0} {
-        lappend argv ".*"
+	lappend argv ".*"
 }
 
 foreach pname $argv {
-
-if {[catch {set res [mportsearch "^${pname}\$"]} result]} {
-	puts "port search failed: $result"
-	exit 1
-}
-
-foreach {name array} $res {
-	global prefix
-	array unset portinfo
-	array set portinfo $array
-
-	if {![info exists portinfo(porturl)]} {
-		puts stderr "Internal error: no porturl for $name"
-		continue
+	if {[catch {set res [mportsearch "^${pname}\$"]} result]} {
+		puts "port search failed: $result"
+		exit 1
 	}
-	
-	set pkgbase "/darwinports/pkgs/"
-	set mpkgbase "/darwinports/mpkgs/"
-	set porturl $portinfo(porturl)
-	set prefix "/opt/local"
 
-	# Skip up-to-date packages
-	if {[regsub {^file://} $portinfo(porturl) "" portpath]} {
-		if {[info exists portinfo(name)] &&
-			[info exists portinfo(version)] &&
-			[info exists portinfo(categories)]} {
-			set portname $portinfo(name)
-			set portversion $portinfo(version)
-			set category [lindex $portinfo(categories) 0]
-			set mpkgfile ${mpkgbase}/${category}/${portname}-${portversion}.mpkg/Contents/Info.plist
-			if {[file readable $mpkgfile] && ([file mtime ${mpkgfile}] > [file mtime ${portpath}/Portfile])} {
-				puts stderr "Skipping ${portname}-${portversion}; meta-package is up to date."
-				continue
+	foreach {name array} $res {
+		global prefix
+		array unset portinfo
+		array set portinfo $array
+
+		if {![info exists portinfo(porturl)]} {
+			puts stderr "Internal error: no porturl for $name"
+			continue
+		}
+
+		set pkgbase "/darwinports/pkgs/"
+		set mpkgbase "/darwinports/mpkgs/"
+		set porturl $portinfo(porturl)
+		set prefix "/opt/local"
+
+		# Skip up-to-date packages
+		if {[regsub {^file://} $portinfo(porturl) "" portpath]} {
+			if {[info exists portinfo(name)] &&
+				[info exists portinfo(version)] &&
+				[info exists portinfo(categories)]} {
+				set portname $portinfo(name)
+				set portversion $portinfo(version)
+				set category [lindex $portinfo(categories) 0]
+				set mpkgfile ${mpkgbase}/${category}/${portname}-${portversion}.mpkg/Contents/Info.plist
+				if {[file readable $mpkgfile] && ([file mtime ${mpkgfile}] > [file mtime ${portpath}/Portfile])} {
+					puts stderr "Skipping ${portname}-${portversion}; meta-package is up to date."
+					continue
+				}
 			}
 		}
-	}
-	
-	# Skipt packages which previously failed
-		
-	# Building the mpkg:
-	# - create an mpkg skeleton
-	# - copy dependent pkgs into Contents/Resources directory
 
-	set portname ""
-	set portversion ""
-	set description ""
-	set long_description ""
-	set homepage ""
-	set category ""
+		# Skipt packages which previously failed
 
-	if {[info exists portinfo(name)]} {	set portname $portinfo(name) }
-	if {[info exists portinfo(version)]} { set portversion $portinfo(version) }
-	if {[info exists portinfo(description)]} { set description $portinfo(description) }
-	if {[info exists portinfo(long_description)]} { set long_description $portinfo(long_description) }
-	if {[info exists portinfo(homepage)]} { set homepage $portinfo(homepage) }
-	if {[info exists portinfo(categories)]} { set category [lindex $portinfo(categories) 0] }
-	if {[info exists portinfo(maintainers)]} { set maintainers $portinfo(maintainers) }
-	
-	puts "meta-packaging ${category}/${portname}-${portversion}"
+		# Building the mpkg:
+		# - create an mpkg skeleton
+		# - copy dependent pkgs into Contents/Resources directory
 
-	set mpkgpath "${mpkgbase}/${category}/${portname}-${portversion}.mpkg"
+		set portname ""
+		set portversion ""
+		set description ""
+		set long_description ""
+		set homepage ""
+		set category ""
 
-	if {[catch {system "mkdir -p -m 0755 ${mpkgpath}/Contents/Resources"} error]} {
-		puts stderr "Internal error: $error"
-	}
-	
-	# list of .pkg names for dependencies,
-	# built up by copy_package_if_available, and used in the Info.plist
-	set dependencies {}
-	set result [copy_package_if_available ${portname} $pkgbase "${mpkgpath}/Contents/Resources/"]
-	if {$result == -1} {
-		puts stderr "aborting; one or more dependencies was missing."
-		if {[catch {system "rm -R ${mpkgpath}"} error]} {
+		if {[info exists portinfo(name)]} {	set portname $portinfo(name) }
+		if {[info exists portinfo(version)]} { set portversion $portinfo(version) }
+		if {[info exists portinfo(description)]} { set description $portinfo(description) }
+		if {[info exists portinfo(long_description)]} { set long_description $portinfo(long_description) }
+		if {[info exists portinfo(homepage)]} { set homepage $portinfo(homepage) }
+		if {[info exists portinfo(categories)]} { set category [lindex $portinfo(categories) 0] }
+		if {[info exists portinfo(maintainers)]} { set maintainers $portinfo(maintainers) }
+
+		puts "meta-packaging ${category}/${portname}-${portversion}"
+
+		set mpkgpath "${mpkgbase}/${category}/${portname}-${portversion}.mpkg"
+
+		if {[catch {system "mkdir -p -m 0755 ${mpkgpath}/Contents/Resources"} error]} {
 			puts stderr "Internal error: $error"
 		}
-		continue
-	} else {
-		set result [lsort -uniq $result]
-		eval "lappend dependencies $result"
-	}
-	
-	#
-	# Begin quote from portmpkg.tcl
-	#
-    write_PkgInfo ${mpkgpath}/Contents/PkgInfo
-    mpkg_write_info_plist ${mpkgpath}/Contents/Info.plist $portname $portversion $prefix $dependencies
-    write_description_plist ${mpkgpath}/Contents/Resources/Description.plist $portname $portversion $description
-    # long_description, description, or homepage may not exist
-    foreach variable {long_description description homepage} {
-	if {![info exists $variable]} {
-	    set pkg_$variable ""
-	} else {
-	    set pkg_$variable [set $variable]
-	}
-    }
-    write_welcome_html ${mpkgpath}/Contents/Resources/Welcome.html $portname $portversion $pkg_long_description $pkg_description $pkg_homepage
-    file copy -force -- /opt/local/share/darwinports/resources/port1.0/package/background.tiff \
-			${mpkgpath}/Contents/Resources/background.tiff
-	#
-	# End quote from portmpkg.tcl
-	#
-}
 
+		# list of .pkg names for dependencies,
+		# built up by copy_package_if_available, and used in the Info.plist
+		set dependencies {}
+		set result [copy_package_if_available ${portname} $pkgbase "${mpkgpath}/Contents/Resources/"]
+		if {$result == -1} {
+			puts stderr "aborting; one or more dependencies was missing."
+			if {[catch {system "rm -R ${mpkgpath}"} error]} {
+				puts stderr "Internal error: $error"
+			}
+			continue
+		} else {
+			set result [lsort -uniq $result]
+			eval "lappend dependencies $result"
+		}
+
+		#
+		# Begin quote from portmpkg.tcl
+		#
+		write_PkgInfo ${mpkgpath}/Contents/PkgInfo
+		mpkg_write_info_plist ${mpkgpath}/Contents/Info.plist $portname $portversion $prefix $dependencies
+		write_description_plist ${mpkgpath}/Contents/Resources/Description.plist $portname $portversion $description
+		# long_description, description, or homepage may not exist
+		foreach variable {long_description description homepage} {
+			if {![info exists $variable]} {
+				set pkg_$variable ""
+			} else {
+				set pkg_$variable [set $variable]
+			}
+		}
+		write_welcome_html ${mpkgpath}/Contents/Resources/Welcome.html $portname $portversion $pkg_long_description $pkg_description $pkg_homepage
+		file copy -force -- /opt/local/share/darwinports/resources/port1.0/package/background.tiff \
+			${mpkgpath}/Contents/Resources/background.tiff
+		#
+		# End quote from portmpkg.tcl
+		#
+	}
 }
 # end foreach pname
