@@ -1,6 +1,5 @@
 /*
  * md_wrappers.h
- * $Id: md_wrappers.h 79593 2011-06-19 20:36:07Z jmr@macports.org $
  *
  * Copyright (c) 2005 Paul Guyot <pguyot@kallisys.net>.
  * All rights reserved.
@@ -79,5 +78,15 @@ static char *algo##File(const char *filename, char *buf)		\
     return algo##End(&ctx, buf);						\
 }
 
+#define CHECKSUMData(algo, ctxtype)						\
+static char *algo##Data(const u_char *str, u_int32_t len, char *buf)		\
+{														\
+    ctxtype ctx;										\
+														\
+    algo##Init(&ctx);									\
+    algo##Update(&ctx,str,len);					        \
+    return algo##End(&ctx, buf);						\
+}
+
 #endif
 /* _MD_WRAPPERS_H */
