@@ -89,7 +89,7 @@ proc portsandbox::set_profile {target} {
         lappend allow_dirs $portdbpath/home/Library/Developer/Xcode \
                            $portdbpath/home/Library/Caches \
                            $portdbpath/home/Library/org.swift.swiftpm \
-                           $portdbpath/home/.swiftpm \
+                           $portdbpath/home/.swiftpm
         # explicitly whitelist source dir to work around problems building
         # Xcode projects in-source
         lappend allow_dirs $worksrcpath
@@ -163,9 +163,8 @@ proc portsandbox::set_profile {target} {
         }
     }
 
-    if {${build.type} eq "xcode"} {
+    if {${os.major} >= 10 && ${build.type} eq "xcode"} {
         # let Xcode create jobs (FIXME: narrow allowed commands)
-        append portsandbox_profile "\
-(allow job-creation)"
+        append portsandbox_profile " (allow job-creation)"
     }
 }
