@@ -47,8 +47,8 @@ options unload.asroot
 set_ui_prefix
 
 proc portunload::unload_main {args} {
-    global UI_PREFIX prefix subport sudo_user
-    set launchctl_path ${portutil::autoconf::launchctl_path}
+    global UI_PREFIX prefix subport sudo_user \
+           portutil::autoconf::launchctl_path
 
     portstartupitem::foreach_startupitem {
         if {$si_install} {
